/*
 * @(#)XmlFilter.java 1.0    1999/14/09
 *
 * Copyright (c) 1998, 1999 Eindhoven University of Technology. All Rights Reserved.
 *
 * This software is proprietary information of the Eindhoven University
 * of Technology. It may be used according to the GNU license.
 *
 */

package AHA;

import XmlUtils.*;
import Expression.*;
import java.io.*;
import java.util.Hashtable;
import java.util.Enumeration;
import org.w3c.dom.*;


	/**
	 * XmlFilter is used for parsing XML documents, and it returns a HTML representation of the data
	 */

public class XmlFilter {
	// debug is used for showing debug information
	private boolean debug = false;

	// standard Object container in the Adaptive Hypermedia system
	protected AHA AHA;
	protected PrintWriter out;
	protected Hashtable Knowledge;

	// Three names for the same document (absolute, relative, without directory)
	protected String FullDocName;
	protected String DocName;
	protected String ShortDocName;


	// XML Elements in an XML document
	private final String ifstring = "if";
	private final String headerstring = "header";
	private final String footerstring = "footer";
	private final String printattr = "print";
	private final String fileattr = "file";
	private final String htmlstring = "htmlcode";
	private final String anchorstring = "A";

	/**
	 * The default constructor is forbidden
	 * @exception AhaException Always thrown because this constructor is not valid
	 */
//	public XmlFilter() throws AhaException {
//		throw new AhaException("Invalid call to default XmlFilter constructor.");
//	}


	/** 
	 * This constructor is for base construction
	 * @param AHA the AHA context
	 */
	public XmlFilter(AHA AHA) {
		this.AHA = AHA;
		this.out = AHA.getWriter();
		this.Knowledge = AHA.getUserModel().getKnowledge();
	}
	
	/** 
	 * This constructor may be used for outside calls
	 * @param AHA pass the general AHA object
	 * @param FullDocName pass the exact url for the datadocument
	 * @param DocName the short name of the document
	 */
	public XmlFilter(AHA AHA, String FullDocName, String DocName){
		this(AHA);
		this.FullDocName = FullDocName;
		this.DocName = DocName;
		println("XmlFilter: "+FullDocName);
		try {
			ShortDocName = new Derive(FullDocName).getDoc();
		} catch (AhaException ahae) {
			ShortDocName = FullDocName;
		}
	}

	/**
	 * Debug information
	 */
	public void println(Object s) { if (debug) System.out.println(s.toString());}
	public void print(Object s) { if (debug) System.out.print(s.toString());}

	/**
	 * test an expression to the users knowledge
	 * @param s the expression to be checked
	 */
	public boolean teststr(String s) {
		try {
			Expr expr = new Expr().parseTerm(s);
			boolean b = expr.eval(Knowledge);
			println("Expr: "+s+"   ("+expr.toString()+") ==  "+b);
			return b;
		} catch (ExprException ee) {
			System.out.println("XmlFilter Error in expression: "+s+" (not parseable).");
			return true;	// default???? CHANGE
		}
	}	

	/**
	 * Node is an if-tag. The expression is given as an attribute "expr".
	 * @param n The if tag
	 */
	public boolean teststr(Node n) {
		Element e = (Element)n;
		return teststr(e.getAttribute("expr"));
	}

	/**
	 * Is this node an if-tag?
	 */
	public boolean isIfStatement(Node node) {
		return node.getNodeName().equals(ifstring);
	}

	/** 
	 * Is this node a header tag?
	 */
	public boolean isHeaderStatement(Node node) {
		return node.getNodeName().equals(headerstring);
	}

	/** 
	 * Is this node a footer tag?
	 */
	public boolean isFooterStatement(Node node) {
		return node.getNodeName().equals(footerstring);
	}

	/**
	 * Is this node a htmltag?
	 */
	public boolean isHtmlCode(Node node) {
		return node.getNodeName().equals(htmlstring);
	}

	/**
	 * Is this node an anchor tag?
	 */
	public boolean isAnchorStatement(Node node) {
		return node.getNodeName().toUpperCase().equals(anchorstring);
	}

	/**
	 * Is this node a block tag?
	 */
	public boolean isBlockNode(Node node) {
		return node.getNodeName().equals("block"); //blockstring);
	}

	/**
	 * Maybe this is only used for testing
	 */
	public StringBuffer printAttr(Node node1) {
		StringBuffer sb = new StringBuffer();
		if (node1 instanceof Element) {
			Element node = (Element)node1;
			if (isIfStatement(node)) {
				String expr = node.getAttribute("expr");
				sb.append("IF "+expr+" = "+teststr(expr));
			}
		}
		//println(sb);
		return new StringBuffer();
	}

	/**
	 * This method gets an if-tag and depending on the expression returns a block
	 * if if-expr=true --> first block	(item(0))
	 * if if-expr=false --> second block if any	(item((1))
	 */
	public StringBuffer makeIfChoice(Node node) throws AhaException {

		StringBuffer sb = new StringBuffer();
		NodeList nodelist = node.getChildNodes();
		boolean b = teststr(node);
		if (b) {
			sb.append(printNode(nodelist.item(0),true));
		}
		if (!b && nodelist.getLength()==2) {
			sb.append(printNode(nodelist.item(1),true));
		}
		return sb;
	}
		

	/**
	 * This method takes care of writing headers and footers,
	 * which consists of calling to the Messages object
	 */
	public StringBuffer doHeaderFooter(Node node) throws AhaException{
		StringBuffer sb = new StringBuffer();
		boolean print = ((Element)node).getAttribute(printattr).equals("yes");
		String filename ="";
		filename = ((Element)node).getAttribute(fileattr);
		println("header or footer filename = "+filename);
		if (filename.equals("")) filename = null;
		if (isHeaderStatement(node)) {
			if (print) 
				sb.append(AHA.getMessages().printBodyTag());
			if (filename !=null) {
				try {
					HeaderFooterFilter hff = new HeaderFooterFilter(AHA, filename, DocName);
					sb.append(hff.makeHTML());
				} catch (AhaException ahae) {
					System.out.println("Error reading headerfile: "+filename+". This file was called by "+DocName+" from course "+AHA.getCourse()+".");
				}
			}
		}
		if (isFooterStatement(node)) {
			if (filename != null) {
				try {
					HeaderFooterFilter hff = new HeaderFooterFilter(AHA, filename, DocName);
					sb.append(hff.makeHTML());
				} catch (AhaException ahae) {
					System.out.println("Error reading footerfile: "+filename+". This file was called by "+DocName+" from course "+AHA.getCourse()+".");
				}
			}
			if (print) 
				sb.append(AHA.getMessages().printEndBodyTag());
		}
		return sb;
	}	

	/**
	 * This returns a string containing an anchor
	 */
	public String filterAnchorNode(Node node) throws AhaException{
		// here the anchor (<a>) tag can be changed
		try {
			Link link = new Link(AHA, node);
			return link.toString();
		}catch (Exception e) {
			System.out.println(e.toString());
		}
		return "BURP";
	}


	/**
	 * Get the (html) representation of this node
	 * @param b if b then generate something, else generate nothing
	 */	
	public StringBuffer printNode(Node node, boolean b) throws AhaException {
		StringBuffer sb = new StringBuffer();
		if (b) {
			String name = node.getNodeName();
			String value = node.getNodeValue();
			//println("name="+name+" value="+value);
			if (isIfStatement(node)){ 
				sb.append(printAttr(node)+"\n");
				sb.append(makeIfChoice(node));
			}
			else if (isHeaderStatement(node) || isFooterStatement(node)) {
					sb.append(doHeaderFooter(node));
			} else  if (isAnchorStatement(node)) {
				//System.out.println("Anchor: "+node.toString());
				sb.append(filterAnchorNode(node).toString());
			}
			else if (isBlockNode(node)) {
				if (node.hasChildNodes()) {
					sb.append(printChildNodes(node,true));
				}
			} else {	// htmlcode
				//System.out.println(node.toString());
				if (node instanceof Text) {
					//System.out.print("Name:"+name+"="+node.toString().trim());
				}
				sb.append(node.toString());
			}
		}
		return sb;
	}

	/**
	 * Get the (html) representation of the childs of this node
	 * @param b pass this boolean to printNode
	 */
	public StringBuffer printChildNodes(Node nodec, boolean b) throws AhaException {
		StringBuffer sb = new StringBuffer();
		NodeList nodelist = nodec.getChildNodes();
		for (int i=0; i<nodelist.getLength(); i++) {
			Node node = nodelist.item(i);
			sb.append(printNode(node,b));
		}
		return sb;
	}

	/**
	 * prints out the html to AHA.getWriter
	 */
	public void run() throws AhaException{
		try {
			XmlReader xml = new XmlReader(FullDocName);

			Element el = xml.openFile();
			StringBuffer buffer = printChildNodes(el,true);
			out.println(buffer);
			out.flush();

		} catch (Exception e) {
			System.out.println("Error reading file: "+FullDocName);
			
			Messages m = AHA.getMessages();

			m.Fatal("Error reading file: "+ShortDocName+" from course "+AHA.getCourse()+". Please notify the author.");
			//throw new AhaException("Error: "+e.toString());
		}
	}

}	